@@ -395,6 +395,8 @@ def first_run (distances, distances_orig_all, distances_orig, vcf_path_clust, vc
 	distance_cut = bincenters2[interval_line]
 	sigValue = 0.01
 	distance_cut, q_val, avgSim, stdevSim, upper_CI_refined, lower_CI_refined = refineIMD (distances, distances_orig, y2, avg_bin_counts, interval_line, bincenters2[interval_line], bincenters2[interval_line+1], CI, lower_CI, upper_CI, sigValue)
+	if distance_cut > 10000:
+		distance_cut = 10000
 	# for x in distances_orig_all:
 	# 	# print(int(x[0]))
 	# 	# print(chromLengths[genome][x[2]])
@@ -422,11 +424,13 @@ def first_run (distances, distances_orig_all, distances_orig, vcf_path_clust, vc
 			# 	nonClustered_muts = [x[1:] + [x[0]] for x in distances_orig_all if x[1:] + [x[0]] not in clustered_muts]				
 			
 			# if len(imds_corrected) > 0:
-			clustered_muts = [x[1:] + [x[0]] for x in distances_orig_all if int(x[0]) <= distance_cut or (regions[catch(x, regions, chromLengths, genome, imds_corrected[x[1]])] - (int(x[3]) + chromLengths[genome][x[2]]) < windowSize and int(x[0]) < imds_corrected[x[1]][regions[catch(x, regions, chromLengths, genome, imds_corrected[x[1]])]])]
-			nonClustered_muts = [x[1:] + [x[0]] for x in distances_orig_all if x[1:] + [x[0]] not in clustered_muts]
-			# else:
-			# 	clustered_muts = [x[1:] + [x[0]] for x in distances_orig_all if int(x[0]) <= distance_cut ]
-			# 	nonClustered_muts = [x[1:] + [x[0]] for x in distances_orig_all if x[1:] + [x[0]] not in clustered_muts]	
+			if len(regions) > 0:
+				clustered_muts = [x[1:] + [x[0]] for x in distances_orig_all if int(x[0]) <= distance_cut or (regions[catch(x, regions, chromLengths, genome)] - (int(x[3]) + chromLengths[genome][x[2]]) < windowSize and cutoffCatch(x, imds_corrected, regions, catch(x, regions, chromLengths, genome), distance_cut))]# int(x[0]) < imds_corrected[x[1]][regions[catch(x, regions, chromLengths, genome, imds_corrected[x[1]])]]))]
+				# clustered_muts = [x[1:] + [x[0]] for x in distances_orig_all if int(x[0]) <= distance_cut or (regions[catch(x, regions, chromLengths, genome, imds_corrected[x[1]])] - (int(x[3]) + chromLengths[genome][x[2]]) < windowSize and cutoffCatch(int(x[0]) < imds_corrected[x[1]][regions[catch(x, regions, chromLengths, genome, imds_corrected[x[1]])]]))]
+				nonClustered_muts = [x[1:] + [x[0]] for x in distances_orig_all if x[1:] + [x[0]] not in clustered_muts]
+			else:
+				clustered_muts = [x[1:] + [x[0]] for x in distances_orig_all if int(x[0]) <= distance_cut ]
+				nonClustered_muts = [x[1:] + [x[0]] for x in distances_orig_all if x[1:] + [x[0]] not in clustered_muts]	
 
 
 		else:
@@ -471,12 +475,20 @@ def first_run (distances, distances_orig_all, distances_orig, vcf_path_clust, vc
 
 def catch (x, regions, chromLengths, genome, imds_corrected=None):
 	bisectPos = bisect.bisect_left(regions, int(x[3]) + chromLengths[genome][x[2]])
-	if bisectPos == len(regions):
+	if bisectPos >= len(regions):
 		bisectPos -= 1
-	if bisectPos > len(imds_corrected):
-		bisectPos = len(imds_corrected)-1
+	# if bisectPos >= len(imds_corrected):
+	# 	bisectPos = len(imds_corrected)-1
 	return(bisectPos)
 
+def cutoffCatch (x, imds_corrected, regions, bisectPos, distance_cut):
+	if x[1] not in imds_corrected:
+		return (int(x[0]) < distance_cut)
+	else:
+		if regions[bisectPos] not in imds_corrected[x[1]]:
+			return (int(x[0]) < distance_cut)
+		else:
+			return (int(x[0]) < imds_corrected[x[1]][regions[bisectPos]])
 
 
 def densityCorrectionOriginal (densityMuts, densityMutsSim, binsDensity):
@@ -530,7 +542,7 @@ def densityCorrection (densityMuts, densityMutsSim, windowSize):
 	Returns:
 	   correctionFolds	->	the cumulative genomic position of regions that have a higher mutation density than what is expected by chance (list)
 	'''
-	threshold = 1.25
+	threshold = 9.0
 	maxDistance = max(max(densityMuts), max([int(densityMutsSim[x][-1]) for x in densityMutsSim])) + windowSize
 	simDensities = []
 	for x in densityMutsSim:
