@@ -997,7 +997,7 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, chromLengths,
 	'''
 	aggregated = False
 	chrom_based = False
-
+	windowSize = 10000000
 	path_suffix = ''
 	if chrom_based_IMD:
 		path_suffix = '_chrom'
@@ -1169,9 +1169,9 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, chromLengths,
 		count = 1
 		for sample in sorted(samplesSet, key=natural_sortkey):
 			chrom_startsAll = {}
-			if correction:
-				# regions = list(imdsDataSample_corrected[sample.split("_")[0]].keys())
-				regions = list(imdsDataSample_corrected[sample.split(".")[0]].keys())
+			# if correction:
+			# 	# regions = list(imdsDataSample_corrected[sample.split("_")[0]].keys())
+			# 	regions = list(imdsDataSample_corrected[sample.split(".")[0]].keys())
 			zorderPlot=0
 			# falsePositives[sample.split("_")[0]] = []
 			# falsePositives_1000[sample.split("_")[0]] = []
@@ -1236,7 +1236,6 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, chromLengths,
 							starts = pos.loc[(sample, chrom), 'start']
 							IMDsPlotRecorded = pos.loc[(sample, chrom), 'IMDplot']
 							imds_recorded = pos.loc[(sample, chrom), 'IMD']
-							
 							if type(starts) == np.int64:
 								starts = [starts]
 								imds_recorded = [imds_recorded]
@@ -1259,24 +1258,27 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, chromLengths,
 							imds = [y-x for x,y in zip(starts, starts[1:])]
 							minIMDs = [min(x,y) for x,y in zip([float('inf')] + imds, imds + [float('inf')])]							
 							if not aggregated:
+								# if subclass == "Class III":
+								# 	if chrom_based_IMD:
+								# 		try:
+								# 			plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split(".")[0]][chrom] or y < 10000]
+								# 			# plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]][chrom] or y < 10000]
+								# 		except:
+								# 			plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsDataSample[sample.split(".")[0]] or y < 10000]
+								# 			# plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsDataSample[sample.split("_")[0]] or y < 10000]
+								# 	else:
+								# 		plotIMDs = [[x,y] for x,y in zip(starts, imds_recorded)]# if y <= imdsData[sample.split("_")[0]] or y < 10000]
+								# 		# plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]] or y < 10000]
+								# else:
+								# 	# plotIMDs = [[x,y] for x,y in zip(starts, imds) if y <= imdsDataSample[sample.split(".")[0]] or y < 10000]
+								# 	plotIMDs = [[x,y] for x,y in zip(starts, minIMDs)]
 								if subclass == "Class III":
-									if chrom_based_IMD:
-										try:
-											plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split(".")[0]][chrom] or y < 10000]
-											# plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]][chrom] or y < 10000]
-										except:
-											plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsDataSample[sample.split(".")[0]] or y < 10000]
-											# plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsDataSample[sample.split("_")[0]] or y < 10000]
-									else:
-										plotIMDs = [[x,y] for x,y in zip(starts, imds_recorded)]# if y <= imdsData[sample.split("_")[0]] or y < 10000]
-										# plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]] or y < 10000]
+									plotIMDs = [[x,int(y)] for z,(x,y) in enumerate(zip(starts, imds_recorded)) if y != 'c' and (z not in newGroup or len(starts) == 1 or len(starts) == len(newGroup))]
 								else:
-									# plotIMDs = [[x,y] for x,y in zip(starts, imds) if y <= imdsDataSample[sample.split(".")[0]] or y < 10000]
-									plotIMDs = [[x,y] for x,y in zip(starts, minIMDs)]
-
-								# plotIMDs = [[x,int(y)] for x,y in zip(starts, IMDsPlotRecorded) if y != 'c' ]
-								plotIMDs = [[x,int(y)] for z,(x,y) in enumerate(zip(starts, IMDsPlotRecorded)) if y != 'c' and (z not in newGroup or len(starts) == 1 or len(starts) == len(newGroup))]
-
+									# plotIMDs = [[x,int(y)] for x,y in zip(starts, IMDsPlotRecorded) if y != 'c' ]
+									plotIMDs = [[x,int(y)] for z,(x,y) in enumerate(zip(starts, IMDsPlotRecorded)) if y != 'c' and (z not in newGroup or len(starts) == 1 or len(starts) == len(newGroup))]
+								if len(plotIMDs) == 0:
+									continue
 								plotX, plotY = zip(*plotIMDs)
 								newMax = max(plotY)
 								if chrom_based_IMD:
